class Timetable < ApplicationRecord
  belongs_to :user, inverse_of: :timetables
  belongs_to :chosen_item, inverse_of: :timetables

  validates :user_id, presence: true
  validates :chosen_item_id, presence: true
  validates :job_type, presence: true
  validates :status, presence: true
  validates :hours, presence: true, numericality: { only_integer: true }
  validates :minutes, presence: true, numericality: { only_integer: true }

  def self.last_two_days
    where created_at: (Date.today - 2.days)..Time.now
  end

  def status_enum
    # Array holding all the roles
    status = %i[
      suspended
      finished
    ]
    # Do not EDIT below this line
    status.map {|s| I18n.t("status.#{s.to_sym}")}
  end

  def job_type_enum
    # Array holding all the roles
    statuses = %i[
      internal_labour
      external_labour
      internal_paint
      external_paint
      upholstery
      storage
      construction_yard
    ]
    # Do not EDIT below this line
    statuses.map {|s| I18n.t("statuses.#{s.to_sym}")}
  end

  def commission
    chosen_item.chosen_furniture.chosen_room.chosen_deck.commission.code rescue ""
  end
  def deck
    chosen_item.chosen_furniture.chosen_room.chosen_deck.deck.name rescue ""
  end
  def room
    chosen_item.chosen_furniture.chosen_room.room.name rescue ""
  end
  def furniture
    chosen_item.chosen_furniture.furniture.name rescue ""
  end
  def item
    chosen_item.furniture_item.item.name rescue ""
  end

  def display_name
    "#{user.username} (#{hours}:#{minutes}): #{[commission, deck, room, furniture, item].compact.join(" - ")}" rescue ""
  end

  rails_admin do
    navigation_label I18n.t("admin.tools.label")
    navigation_icon 'fa fa-calendar'
    weight 13

    export do
      field :user, :string do
        export_value do
          value.email if value #value is an instance of Teacher
        end
      end
      field :commission
      field :deck
      field :room
      field :furniture
      field :item
      field :chosen_item, :string do
        label do
          "Codice a Barre"
        end
        export_value do
          value.barcode if value #value is an instance of ChosenItem
        end
      end
      field :job_type
      field :status
      field :hours do
        label do
          "Ore Lavorate"
        end
      end
      field :minutes do
        label do
          "Minuti Lavorati"
        end
      end
    end

    list do
      scopes [:last_two_days, nil]
      field :user do
        searchable [{User => :username}]
        visible do
          !bindings[:view].current_user.has_role?(:workers)
        end
      end

      field :chosen_item
      field :job_type
      field :status
      field :hours
      field :minutes
      field :created_at
    end

    edit do
      field :chosen_item
      field :job_type
      field :status
      field :hours
      field :minutes
      field :notes
      # Preset user to current user
      field :user do
        visible do
          # If it's a worker just show the user_id part, and hide the !user_id
          bindings[:view].current_user.admin?
        end
      end
      field :user_id, :hidden do
        label "&nbsp;".html_safe
        help "&nbsp;".html_safe
        visible do
          # If it's a worker just show the user_id part, and hide the !user_id
          bindings[:view].current_user.has_role?(:workers) && !bindings[:view].current_user.admin?
        end
        # TODO: Migliorare questo dato per evitare che venga sovrascritto l'autore originale in edit successivi da parte di altri
        default_value do
          bindings[:view]._current_user.id
        end
      end
    end
    create do
      field :chosen_item
      field :job_type
      field :status
      field :hours
      field :minutes
      field :notes
      # Preset user to current user
      field :user do
        visible do
          # If it's a worker just show the user_id part, and hide the !user_id
          bindings[:view].current_user.admin?
        end
      end
      field :user_id, :hidden do
        label "&nbsp;".html_safe
        help "&nbsp;".html_safe
        visible do
          # If it's a worker just show the user_id part, and hide the !user_id
          bindings[:view].current_user.has_role?(:workers) && !bindings[:view].current_user.admin?
        end
        # TODO: Migliorare questo dato per evitare che venga sovrascritto l'autore originale in edit successivi da parte di altri
        default_value do
          bindings[:view]._current_user.id
        end
      end
    end
  end
end
