class Workflow < ApplicationRecord
  belongs_to :chosen_item, inverse_of: :workflows
  belongs_to :user, inverse_of: :workflows
  belongs_to :third_party, class_name: 'User', inverse_of: :assigneds

  validates :statuses, presence: true
  validates :chosen_item, presence: true
  validates :user, presence: true

  default_scope {
    joins({
      chosen_item: [
        {furniture_item: :item},
        {
          chosen_furniture: [
            :furniture,
            {
              chosen_room: [
                :room,
                {
                  chosen_deck: [
                    :deck,
                    :commission
                  ]
                }
              ]
            }
          ]
        }
      ]
    })
  }

  # useful for filters
  def commission
    chosen_item.chosen_furniture.chosen_room.chosen_deck.commission.code rescue "-"
  end

  def deck
    chosen_item.chosen_furniture.chosen_room.chosen_deck.deck.name rescue "-"
  end

  def room
    chosen_item.chosen_furniture.chosen_room.room.name rescue "-"
  end

  def furniture
    chosen_item.chosen_furniture.furniture.name rescue "-"
  end

  def item
    chosen_item.furniture_item.item.name rescue "-"
  end

  def barcode
    chosen_item.barcode
  end

  STAT = %i[
    internal_labour
    external_labour
    internal_paint
    external_paint
    upholstery
    storage
    construction_yard
  ]

  STAT_TRANSLATED = STAT.each_with_index.map {|a,i| [I18n.t("statuses.#{a.to_sym}"), (i+1).to_s]}

  def self.stats
    STAT
  end

  def self.stats_translated
    STAT_TRANSLATED
  end

  def statuses_enum
    # Array holding all the roles
    # Do not EDIT below this line
    # statuses.map {|s| I18n.t("statuses.#{s.to_sym}")}
    STAT_TRANSLATED
  end

  def stat
    I18n.t "statuses.#{STAT[(statuses.to_i-1)].to_sym}"
  end

  def display_name
    # [commission, deck, room, furniture, item].compact.join(" - ")
    chosen_item.barcode
  end

  rails_admin do
    navigation_label I18n.t("admin.tools.label")
    navigation_icon 'fa fa-code-fork'
    weight 9
 
    # field :chosen_item
    field :barcode do
      read_only true
      # This is a very sad hack, leveraging on the hackability of 
      # this part of code.
      # I expect to see it explode in a while, trying to search for a solution,
      # a true one, meanwhile... I use the hack (rails_admin just adds a dot 
      # between key and value.. thus rebuilding the string as needed)
      searchable [{ "concat(commissions" => "code,decks.code,chosen_decks.code,rooms.code,chosen_rooms.code,furnitures.code,chosen_furnitures.code,items.code,LPAD(chosen_items.code::text, 5, '0'))"}]
    end 
    field :statuses
    field :third_party do
      searchable [{User => :username}]
      associated_collection_cache_all false  # REQUIRED if you want to SORT the list
      associated_collection_scope do
        # bindings[:object] & bindings[:controller] are available, but not in scope's block!
        workflow = bindings[:object]
        Proc.new { |scope|
          # Looking just for third parties
          scope = scope.where(third_party: true)
        }
      end
    end
    field :user do
      searchable [{User => :username}]
    end
    field :created_at do
      read_only true
    end  
    edit do
      field :chosen_item
      field :description
    end
    show do
      field :item
      field :description
    end
    list do
      field :commission do
        #column_width 12
        searchable "commissions.code"
        read_only true
        sortable "commissions.code"
        filterable true
      end
      field :deck do
        #column_width 12
        searchable "decks.name"
        sortable "decks.name"
        read_only true
        filterable true
      end
      field :room do
        searchable "rooms.name"
        sortable "rooms.name"
        read_only true
        filterable true
      end
      field :furniture do
        searchable "furnitures.name"
        sortable "furnitures.name"
        read_only true
        filterable true
      end
      field :item do
        searchable "items.name"
        sortable "items.name"
        read_only true
        filterable true
      end
    end

  end
end
